# frozen_string_literal: true

require 'bundler'
require 'bundler/shellsplit_plugin/version'
require 'bundler/installer/gem_installer'
require 'shellwords'

module Bundler
  # Plugin module containing the hotfix for the shellsplit error.
  module ShellsplitPlugin
    # Patch module which will be patched into the gem installer.
    # Adapted from https://github.com/rubygems/bundler/issues/6940#issuecomment-470643613
    module ShellsplitSpecSettings
      def spec_settings
        return unless (settings = super)

        Shellwords.shellsplit settings
      end
    end

    OLDEST_AFFECTED = Gem::Version.new('1.17.3')
    NEWEST_AFFECTED = Gem::Version.new('2.0.2')

    class << self
      def bundler_affected?
        bundler_version = Gem::Version.new(Bundler::VERSION)
        bundler_version >= OLDEST_AFFECTED && bundler_version <= NEWEST_AFFECTED
      end

      def register
        unless bundler_affected?
          Warning.warn('The Bundler::ShellsplitPlugin plugin is intended ' \
                       'for Bundler versions >= 1.17.3 and <= 2.0.2. It ' \
                       'has no effect on other versions, which do not have ' \
                       'the bug it attempts to fix.')
          return
        end

        Bundler::Plugin.add_hook('before-install') do
          Bundler::GemInstaller.prepend ShellsplitSpecSettings
        end
      end
    end
  end
end
